package store

import (
	"bytes"

	"github.com/gnolang/gno/pkgs/std"
	"github.com/gnolang/gno/pkgs/store/types"
)

// Gets the first item.
func First(st Store, start, end []byte) (kv std.KVPair, ok bool) {
	iter := st.Iterator(start, end)
	if !iter.Valid() {
		return kv, false
	}
	defer iter.Close()

	return std.KVPair{Key: iter.Key(), Value: iter.Value()}, true
}

// Gets the last item.  `end` is exclusive.
func Last(st Store, start, end []byte) (kv std.KVPair, ok bool) {
	iter := st.ReverseIterator(end, start)
	if !iter.Valid() {
		if v := st.Get(start); v != nil {
			return std.KVPair{Key: types.Cp(start), Value: types.Cp(v)}, true
		}
		return kv, false
	}
	defer iter.Close()

	if bytes.Equal(iter.Key(), end) {
		// Skip this one, end is exclusive.
		iter.Next()
		if !iter.Valid() {
			return kv, false
		}
	}

	return std.KVPair{Key: iter.Key(), Value: iter.Value()}, true
}
